""" Conveniences for wresting detailed info from the archive MDDB """

import logging
import sys

# pylint: disable=E0401, E0402

import psycopg2 as pg
from pendulum import Duration
from pycapo import CapoConfig
from sqlalchemy.engine import Connection

logger = logging.getLogger("aat_wrest")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))

PENDULUM_FORMAT = "YYYY-MM-DDTHH:mm:ss"
TIME_ZONE = "UTC"


class MDDBConnector:
    """Use this connection to interrogate this science product locator"""

    def __init__(self):
        self.connection = self._connect_to_mddb()

    def _connect_to_mddb(self) -> Connection:
        """
        Establish a DB connection

        :return:
        """
        settings = CapoConfig().settings("metadataDatabase")

        host, dbname = settings["jdbcUrl"].split(":")[2][2:].split("/")
        try:
            conn = pg.connect(
                host=host,
                database=dbname,
                user=settings.jdbcUsername,
                password=settings.jdbcPassword,
            )
            return conn
        except Exception as exc:
            print(f"Unable to connect to database: {exc}")
            raise exc

    def cursor(self):
        return self.connection.cursor()

    def close(self):
        self.connection.close()


def format_interval(interval: Duration) -> str:
    """
    Format this time interval in a human-readable way

    :param interval:
    :return:
    """

    formatted = (
        f"{interval.days} days, {interval.hours} "
        f"hours, "
        f"{interval.minutes} minutes, {interval.remaining_seconds} seconds"
    )
    return formatted
