from datetime import datetime
from unittest.mock import patch

from workspaces.system.schema import AbstractFile

from workspaces.system.services.remote_processing_service import CapoInjector
from workspaces.workflow.schema import WorkflowRequest

"""
tests for remote_processing_service.CapoInjector
"""

expected_view = [
    "edu.nrao.workspaces.WorkflowSettings.serviceUrl",
    "edu.nrao.workspaces.NotificationSettings.serviceUrl",
    "edu.nrao.workspaces.UISettings.serviceUrl",
    "edu.nrao.workspaces.ProcessingSettings.useCasa",
    "edu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing",
]
expected_subspace = [
    "edu.nrao.workspaces.WorkflowSettings.serviceUrl = http://workflow:3456",
    "edu.nrao.workspaces.NotificationSettings.serviceUrl = http://notification:3458",
    "edu.nrao.workspaces.UISettings.serviceUrl = http://localhost:4444/workspaces",
    "edu.nrao.workspaces.ProcessingSettings.useCasa = false",
    "edu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing = /home/casa/packages/pipeline/current",
]

request = WorkflowRequest(
    workflow_request_id=-3,
    workflow_name="vlass_seci",
    argument={"remote": True},
    state="Created",
    results_dir="/i/am/a/path",
    created_at=datetime(2021, 1, 1),
    updated_at=datetime(2021, 1, 1),
)

injector = CapoInjector(request)


class TestCapoInjector:
    def test_is_remote_workflow(self):
        check = injector.is_remote_workflow()
        assert check is True

    def test_identify_subspace(self):
        view = injector.identify_subspace()
        assert view == expected_view

    def test_create_subspace(self):
        subspace = injector.create_subspace()
        assert subspace == expected_subspace

    def test_write_subspace(self):
        abs_file = injector.write_subspace()
        assert isinstance(abs_file, AbstractFile)
        assert abs_file.filename == "docker.properties"
        assert (
            abs_file.content
            == """edu.nrao.workspaces.WorkflowSettings.serviceUrl = http://workflow:3456\nedu.nrao.workspaces.NotificationSettings.serviceUrl = http://notification:3458\nedu.nrao.workspaces.UISettings.serviceUrl = http://localhost:4444/workspaces\nedu.nrao.workspaces.ProcessingSettings.useCasa = false\nedu.nrao.archive.workflow.config.CasaVersions.homeForReprocessing = /home/casa/packages/pipeline/current""".encode()
        )

    @patch("pathlib.Path.write_bytes")
    def test_subspace_to_directory(self, mock_write):
        injector.subspace_to_directory()
        assert mock_write.call_count == 1

    @patch("pathlib.Path.unlink")
    @patch("glob.glob")
    @patch("os.listdir")
    def test_clear_subspace(self, mock_os, mock_glob, mock_remove):
        injector.clear_subspace()
        assert mock_os.call_count == 1
        assert mock_glob.call_count == 1
